package cn.x47.handle;

import cn.x47.model.RIPEntry;
import cn.x47.model.RIPPacket;
import cn.x47.service.RoutingTable;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;

public class RIPServerHandler extends SimpleChannelInboundHandler<RIPPacket> {

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, RIPPacket msg) throws Exception {
        InetSocketAddress senderAddress = msg.getSenderAddress();
        byte version = msg.getVersion();
        System.out.println(msg.toString());

//        if (msg.getCommand() == 2) { // Response
//            for (RIPEntry entry : msg.getEntries()) {
//                // 更新本地路由表
//                updateRoutingTable(entry, senderAddress, version);
//            }
//        } else if (msg.getCommand() == 1) { // Request
//            // 处理请求，发送响应包
//            handleRequest(ctx, msg);
//        }
    }

    private void handleRequest(ChannelHandlerContext ctx, RIPPacket requestPacket) throws Exception {
        List<RIPEntry> requestEntries = requestPacket.getEntries();
        byte version = requestPacket.getVersion();

        boolean isGeneralRequest = false;

        // 检查是否为一般请求
        if (requestEntries.size() == 1) {
            RIPEntry entry = requestEntries.get(0);
            if (entry.getIpAddress().getHostAddress().equals("0.0.0.0") && entry.getMetric() == 16) {
                isGeneralRequest = true;
            }
        }

        RIPPacket responsePacket = new RIPPacket();
        responsePacket.setCommand((byte) 2); // Response
        responsePacket.setVersion(version); // 与请求的版本一致
        responsePacket.setUnused((short) 0);
        List<RIPEntry> responseEntries = new ArrayList<>();

        if (isGeneralRequest) {
            // 一般请求，返回整个路由表
            responseEntries.addAll(getAllRoutingEntries(version));
        } else {
            // 特定请求，逐一处理请求的条目
            for (RIPEntry requestEntry : requestEntries) {
                RIPEntry route = getRoutingEntry(requestEntry.getIpAddress());
                if (route != null) {
                    responseEntries.add(route);
                } else {
                    // 如果没有该路由，返回度量值为 16（无限）
                    RIPEntry unreachableEntry = new RIPEntry();
                    unreachableEntry.setAddressFamily(requestEntry.getAddressFamily());
                    unreachableEntry.setIpAddress(requestEntry.getIpAddress());
                    unreachableEntry.setMetric(16);

                    if (version == 2) {
                        unreachableEntry.setRouteTag((short) 0);
                        unreachableEntry.setSubnetMask(requestEntry.getSubnetMask());
                        unreachableEntry.setNextHop(InetAddress.getByName("0.0.0.0"));
                    }

                    responseEntries.add(unreachableEntry);
                }
            }
        }

        // 构建响应包
        responsePacket.setEntries(responseEntries);

        // 获取发送者的地址
        InetSocketAddress recipient = requestPacket.getSenderAddress();

        // 发送响应包
        ByteBuf buf = ctx.alloc().buffer().order(ByteOrder.BIG_ENDIAN); // 确保使用大端序
        encodeRIPPacket(responsePacket, buf);

        DatagramPacket responseDatagram = new DatagramPacket(buf, recipient);
        ctx.writeAndFlush(responseDatagram);
    }

    private void encodeRIPPacket(RIPPacket msg, ByteBuf buf) throws Exception {
        // 编码命令和版本
        buf.writeByte(msg.getCommand());
        buf.writeByte(msg.getVersion());
        buf.writeShort(0); // unused 字段

        // 编码路由条目
        for (RIPEntry entry : msg.getEntries()) {
            buf.writeShort(entry.getAddressFamily());
            if (msg.getVersion() == 2) {
                buf.writeShort(entry.getRouteTag());
            }
            buf.writeBytes(entry.getIpAddress().getAddress());

            if (msg.getVersion() == 2) {
                buf.writeBytes(entry.getSubnetMask().getAddress());
                buf.writeBytes(entry.getNextHop().getAddress());
            }
            buf.writeInt((int) (entry.getMetric() & 0xFFFFFFFFL));
        }
    }

    private List<RIPEntry> getAllRoutingEntries(byte version) throws Exception {
        List<RIPEntry> entries = new ArrayList<>();
        // 从路由表中获取所有的路由条目
        for (RIPEntry route : RoutingTable.getInstance().getRoutes().values()) {
            RIPEntry entry = new RIPEntry();
            entry.setAddressFamily(route.getAddressFamily());
            entry.setIpAddress(route.getIpAddress());
            entry.setMetric(route.getMetric());

            if (version == 2) {
                entry.setRouteTag(route.getRouteTag());
                entry.setSubnetMask(route.getSubnetMask());
                entry.setNextHop(route.getNextHop());
            }

            entries.add(entry);
        }
        return entries;
    }

    private RIPEntry getRoutingEntry(InetAddress ipAddress) throws Exception {
        RIPEntry route = RoutingTable.getInstance().getRoute(ipAddress);
        if (route != null) {
            RIPEntry entry = new RIPEntry();
            entry.setAddressFamily(route.getAddressFamily());
            entry.setIpAddress(route.getIpAddress());
            entry.setMetric(route.getMetric());

            entry.setRouteTag(route.getRouteTag());
            entry.setSubnetMask(route.getSubnetMask());
            entry.setNextHop(route.getNextHop());

            return entry;
        }
        return null;
    }

    private void updateRoutingTable(RIPEntry entry, InetSocketAddress senderAddress, byte version) throws Exception {
        if (version == 1) {
            // 推断下一跳为发送者的地址
            entry.setNextHop(senderAddress.getAddress());
            // 根据 IP 地址类别推断子网掩码
            InetAddress ipAddress = entry.getIpAddress();
            byte[] addressBytes = ipAddress.getAddress();
            int firstOctet = addressBytes[0] & 0xFF;

            if (firstOctet >= 0 && firstOctet <= 127) {
                // A 类地址
                entry.setSubnetMask(InetAddress.getByName("255.0.0.0"));
            } else if (firstOctet >= 128 && firstOctet <= 191) {
                // B 类地址
                entry.setSubnetMask(InetAddress.getByName("255.255.0.0"));
            } else if (firstOctet >= 192 && firstOctet <= 223) {
                // C 类地址
                entry.setSubnetMask(InetAddress.getByName("255.255.255.0"));
            } else {
                // 非法地址，忽略
                System.out.println("Invalid IP address in RIP Entry: " + ipAddress.getHostAddress());
                return;
            }
        }
        // 实现路由表更新逻辑
        RoutingTable.getInstance().updateRoute(entry);
        System.out.println("Updated route: " + entry.getIpAddress().getHostAddress() +
                " via " + entry.getNextHop().getHostAddress() +
                " metric: " + entry.getMetric());
    }
}
